"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypeormUtilsService = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const typeorm_1 = require("typeorm");
const types_1 = require("../../types");
const utils_1 = require("../../helper/utils");
const constants_1 = require("../../constants");
function isTargetField(relationField, field) {
    return relationField.includes(field);
}
function isRelationField(relationField, field) {
    if (isTargetField(relationField, field))
        return;
    const error = {
        code: 'unrecognized_keys',
        path: ['data', 'relationships'],
        message: `Resource for relation '${field.toString()}' does not exist`,
        keys: [field],
    };
    throw new common_1.BadRequestException([error]);
}
(0, common_1.Injectable)();
let TypeormUtilsService = class TypeormUtilsService {
    repository;
    _currentAlias;
    _relationMetadata = {};
    _relationAlias = {};
    _relationFields;
    _entityMetadata;
    _number = 0;
    constructor(repository) {
        this.repository = repository;
        this._currentAlias = (0, utils_1.snakeToCamel)(repository.metadata.name);
        const relationFields = [];
        for (const metadata of repository.metadata.relations) {
            const propertyName = metadata.propertyName;
            this._relationMetadata[propertyName] = metadata;
            this._relationAlias[propertyName] = (0, utils_1.snakeToCamel)(metadata.inverseEntityMetadata.name);
            relationFields.push(propertyName);
        }
        this._relationFields = relationFields;
        this._entityMetadata = repository.metadata;
    }
    get currentAlias() {
        return this._currentAlias;
    }
    get relationFields() {
        return this._relationFields;
    }
    relationName(relName) {
        return this._relationAlias[relName];
    }
    get currentPrimaryColumn() {
        return this._entityMetadata.primaryColumns[0].propertyName;
    }
    getAliasForRelation(relName) {
        return `${this.currentAlias}__${this._relationAlias[relName]}_${relName}`;
    }
    getRelMetaDataForRelation(relName) {
        return this._relationMetadata[relName];
    }
    getPrimaryColumnForRel(relName) {
        return this._relationMetadata[relName].inverseEntityMetadata
            .primaryColumns[0].propertyName;
    }
    getFilterObject(query, filterType) {
        const { filter } = query;
        if (!filter)
            return null;
        return filter[filterType];
    }
    get number() {
        if (this._number > 1000) {
            this._number = 0;
        }
        this._number++;
        return this._number;
    }
    getParamName(fieldName) {
        return `params_${fieldName}_${this.number}`;
    }
    getAliasPath(fieldName, relname, separator = '.') {
        const alias = relname
            ? this._relationAlias[relname] || relname
            : this.currentAlias;
        return `${alias}${separator}${fieldName}`;
    }
    getSubQueryForManyToMany(fieldName, expression) {
        const metadataRelation = this._relationMetadata[fieldName];
        const relationPrimaryColumn = metadataRelation.inverseEntityMetadata.primaryColumns[0].propertyName;
        const { joinTableName, inverseJoinColumns, joinColumns } = metadataRelation.isManyToManyOwner
            ? metadataRelation
            : metadataRelation.inverseRelation || metadataRelation;
        const { databaseName: queryJoinPropsName } = metadataRelation.isManyToManyOwner
            ? inverseJoinColumns[0]
            : joinColumns[0];
        const { databaseName: selectJoinPropsName } = metadataRelation.isManyToManyOwner
            ? joinColumns[0]
            : inverseJoinColumns[0];
        const alias = this.getAliasPath(this._relationAlias[fieldName], this.currentAlias, '-');
        const selectAlias = this.getAliasPath(selectJoinPropsName, alias);
        const query = this.repository
            .createQueryBuilder()
            .subQuery()
            .select(selectAlias)
            .from(joinTableName, alias)
            .leftJoin(this._relationMetadata[fieldName].inverseEntityMetadata.target, this.getAliasForRelation(fieldName), `${this.getAliasPath(queryJoinPropsName, alias)} = ${this.getAliasPath(relationPrimaryColumn, this.getAliasForRelation(fieldName))}`);
        if (!expression) {
            query.where(`${selectAlias} = ${this.getAliasPath(this.currentPrimaryColumn)}`);
        }
        else {
            for (const i in expression) {
                query[i === '0' ? 'where' : 'andWhere'](expression[i]);
            }
        }
        return query.getQuery();
    }
    getFilterExpressionForTarget(query) {
        const resultExpression = [];
        const filterTarget = this.getFilterObject(query, 'target');
        if (!filterTarget)
            return resultExpression;
        for (const [fieldName, filter] of utils_1.ObjectTyped.entries(filterTarget)) {
            if (!filter)
                continue;
            for (const entries of utils_1.ObjectTyped.entries(filter)) {
                const [operand, value] = entries;
                const valueConditional = operand === types_1.FilterOperand.like ? `%${value}%` : value;
                const fieldWithAlias = this.getAliasPath(fieldName);
                const paramsName = this.getParamName(fieldWithAlias);
                if (!isTargetField(this._relationFields, fieldName)) {
                    if ((operand === types_1.FilterOperand.ne || operand === types_1.FilterOperand.eq) &&
                        (valueConditional === 'null' || valueConditional === null)) {
                        const expression = types_1.OperandMapExpressionForNull[operand].replace(types_1.EXPRESSION, paramsName);
                        resultExpression.push({
                            alias: fieldWithAlias,
                            expression,
                        });
                        continue;
                    }
                    const expression = types_1.OperandsMapExpression[operand].replace(types_1.EXPRESSION, paramsName);
                    resultExpression.push({
                        alias: fieldWithAlias,
                        expression,
                        params: {
                            val: valueConditional,
                            name: paramsName,
                        },
                    });
                    continue;
                }
                const metadataRelation = this._relationMetadata[fieldName];
                const relationTarget = metadataRelation.inverseEntityMetadata.target;
                const relationAlias = this._relationAlias[fieldName];
                const subQuery = this.repository.createQueryBuilder().subQuery();
                const resultOperand = operand === types_1.FilterOperand.eq ? operand : types_1.FilterOperand.ne;
                switch (metadataRelation.relationType) {
                    case 'many-to-many': {
                        const subQuerySql = this.getSubQueryForManyToMany(fieldName);
                        const resultOperand = operand === types_1.FilterOperand.eq ? operand : types_1.FilterOperand.ne;
                        const expression = types_1.OperandsMapExpressionForNullRelation[resultOperand].replace(types_1.EXPRESSION, subQuerySql);
                        resultExpression.push({
                            expression,
                        });
                        break;
                    }
                    case 'one-to-many': {
                        const joinColumn = metadataRelation.inverseSidePropertyPath;
                        const aliasPath = this.getAliasPath(joinColumn, fieldName);
                        const subQuerySql = subQuery
                            .select(aliasPath, joinColumn)
                            .from(relationTarget, relationAlias)
                            .where(`${aliasPath} = ${this.getAliasPath(this.currentPrimaryColumn)}`)
                            .getQuery();
                        const expression = types_1.OperandsMapExpressionForNullRelation[resultOperand].replace(types_1.EXPRESSION, subQuerySql);
                        resultExpression.push({
                            expression,
                        });
                        break;
                    }
                    default: {
                        const expression = types_1.OperandMapExpressionForNull[resultOperand].replace(types_1.EXPRESSION, paramsName);
                        resultExpression.push({
                            alias: fieldWithAlias,
                            expression,
                        });
                    }
                }
            }
        }
        return resultExpression;
    }
    getFilterExpressionForRelation(query) {
        const resultExpression = [];
        const filterRelation = this.getFilterObject(query, 'relation');
        if (!filterRelation)
            return resultExpression;
        for (const [relationField, propsFilter] of utils_1.ObjectTyped.entries(filterRelation)) {
            if (!propsFilter)
                continue;
            if (!isTargetField(this._relationFields, relationField))
                continue;
            const metadataRelation = this._relationMetadata[relationField];
            const conditionalForManyToMany = [];
            for (const [relationFieldProps, filter] of utils_1.ObjectTyped.entries(propsFilter)) {
                if (!filter)
                    continue;
                for (const entries of utils_1.ObjectTyped.entries(filter)) {
                    const [operand, value] = entries;
                    const currentValue = operand === types_1.FilterOperand.like ? `%${value}%` : value;
                    const paramsName = this.getParamName(this.getAliasPath(relationFieldProps.toString(), relationField));
                    let expression;
                    if (value === 'null') {
                        const currentOperand = operand === types_1.FilterOperand.eq
                            ? types_1.FilterOperand.eq
                            : types_1.FilterOperand.ne;
                        expression = types_1.OperandMapExpressionForNull[currentOperand];
                    }
                    else {
                        expression = types_1.OperandsMapExpression[operand].replace(types_1.EXPRESSION, paramsName);
                    }
                    const params = value === 'null'
                        ? undefined
                        : {
                            val: currentValue,
                            name: paramsName,
                        };
                    switch (metadataRelation.relationType) {
                        case 'many-to-many': {
                            conditionalForManyToMany.push({
                                params,
                                conditional: `${this.getAliasPath(relationFieldProps.toString(), this.getAliasForRelation(relationField))} ${expression}`,
                            });
                            break;
                        }
                        default: {
                            resultExpression.push({
                                alias: this.getAliasPath(relationFieldProps.toString(), this.getAliasForRelation(relationField)),
                                expression,
                                selectInclude: relationField,
                                params,
                            });
                        }
                    }
                }
            }
            if (conditionalForManyToMany.length) {
                const expression = conditionalForManyToMany.map((i) => i.conditional);
                const subQuery = this.getSubQueryForManyToMany(relationField, expression);
                const mainExpression = `IN ${subQuery}`;
                const params = conditionalForManyToMany
                    .filter((i) => i.params)
                    .map((i) => i.params);
                resultExpression.push({
                    alias: this.getAliasPath(this.currentPrimaryColumn),
                    expression: mainExpression,
                    paramsForResult: expression,
                    params,
                });
            }
        }
        return resultExpression;
    }
    throwError(message, path, key) {
        const error = {
            code: 'unrecognized_keys',
            path,
            message,
        };
        if (key) {
            error.keys = [key];
        }
        throw new common_1.BadRequestException([error]);
    }
    async *asyncIterateFindRelationships(relationships) {
        for (const entries of utils_1.ObjectTyped.entries(relationships)) {
            const [props, data] = entries;
            isRelationField(this._relationFields, props);
            if (data === undefined)
                continue;
            if (data === null) {
                yield { [props]: null };
                continue;
            }
            const isArray = Array.isArray(data);
            if (isArray && data.length === 0) {
                yield { [props]: [] };
                continue;
            }
            const condition = isArray ? (0, typeorm_1.In)(data.map((i) => i.id)) : (0, typeorm_1.Equal)(data['id']);
            const relationsTypeName = (0, utils_1.kebabToCamel)(isArray ? data[0]['type'] : data['type']);
            const primaryField = this.getPrimaryColumnForRel(props);
            const relationsTarget = this._relationMetadata[props]
                .inverseEntityMetadata.target;
            const result = await this.repository.manager
                .getRepository(relationsTarget)
                .find({
                select: {
                    [primaryField]: true,
                },
                where: {
                    [primaryField]: condition,
                },
            });
            if ((isArray && (result.length === 0 || data.length !== result.length)) ||
                (!isArray && result.length === 0)) {
                const message = isArray
                    ? `Resource '${relationsTypeName}' with ids '${data
                        .map((i) => i.id)
                        .filter((i) => !result.find((r) => r[primaryField] == i))
                        .join(',')}' does not exist`
                    : `Resource '${relationsTypeName}' with id '${data.id}' does not exist`;
                const error = {
                    code: 'invalid_arguments',
                    path: ['data', 'relationships', props.toString()],
                    message,
                };
                throw new common_1.BadRequestException([error]);
            }
            yield { [props]: isArray ? result : result[0] };
        }
    }
    async saveEntityData(target, relationships) {
        if (relationships) {
            for await (const item of this.asyncIterateFindRelationships(relationships)) {
                const [props, type] = utils_1.ObjectTyped.entries(item)[0];
                if (type !== null) {
                    target[props] = type;
                }
                else {
                    target[props] = null;
                }
            }
        }
        const saveData = await this.repository.save(target);
        let saveDataWithRelation = null;
        if (relationships) {
            const queryBuilder = this.repository
                .createQueryBuilder(this.currentAlias)
                .where({
                [this.currentPrimaryColumn]: (0, typeorm_1.Equal)(saveData[this.currentPrimaryColumn]),
            });
            for (const [props] of utils_1.ObjectTyped.entries(relationships)) {
                const currentIncludeAlias = this.getAliasForRelation(props.toString());
                queryBuilder.leftJoinAndSelect(this.getAliasPath(props), currentIncludeAlias);
            }
            saveDataWithRelation = await queryBuilder.getOne();
        }
        return saveDataWithRelation ? saveDataWithRelation : saveData;
    }
    async validateRelationInputData(rel, inputData) {
        const relationMetadata = this._relationMetadata[rel];
        const isArray = Array.isArray(inputData);
        if (['one-to-many', 'many-to-many'].includes(relationMetadata.relationType) &&
            !isArray) {
            const error = {
                code: 'invalid_arguments',
                path: ['data'],
                message: 'Body data should be array',
            };
            throw new common_1.UnprocessableEntityException([error]);
        }
        if (['one-to-one', 'many-to-one'].includes(relationMetadata.relationType) &&
            isArray) {
            const error = {
                code: 'invalid_arguments',
                path: ['data'],
                message: 'Body data should be object',
            };
            throw new common_1.UnprocessableEntityException([error]);
        }
        if (inputData === null) {
            const result = null;
            return result;
        }
        if (isArray && inputData.length === 0) {
            const result = [];
            return result;
        }
        const prepareData = isArray ? inputData : [inputData];
        const errors = [];
        let i = 0;
        const typeName = (0, utils_1.camelToKebab)((0, utils_1.getEntityName)(relationMetadata.inverseEntityMetadata.target));
        for (const prepareItem of prepareData) {
            if (prepareItem.type !== typeName) {
                const path = isArray ? ['data', i.toString()] : ['data'];
                errors.push({
                    code: 'invalid_arguments',
                    path: path,
                    message: `Type should be equal to type of relName: "${rel.toString()}". Type of ${rel.toString()} is "${typeName}" but receive - "${prepareItem.type}"`,
                });
            }
            i++;
        }
        if (errors.length) {
            throw new common_1.UnprocessableEntityException(errors);
        }
        const checkResult = await this.repository.manager
            .getRepository(relationMetadata.inverseEntityMetadata.target)
            .find({
            select: {
                [this.getPrimaryColumnForRel(rel.toString())]: true,
            },
            where: {
                [this.getPrimaryColumnForRel(rel.toString())]: (0, typeorm_1.In)(prepareData.map((i) => i.id)),
            },
        });
        if (checkResult.length === prepareData.length) {
            return (isArray ? inputData.map((i) => i.id) : inputData.id);
        }
        const resulDataMap = checkResult.reduce((acum, item) => {
            acum[item[this.getPrimaryColumnForRel(rel.toString())]] = true;
            return acum;
        }, {});
        i = 0;
        for (const item of prepareData) {
            if (!resulDataMap[item.id]) {
                const path = isArray ? ['data', i.toString(), 'id'] : ['data', 'id'];
                errors.push({
                    code: 'invalid_arguments',
                    path: path,
                    message: `Not exist item "${item.id}" in relation "${rel.toString()}"`,
                });
            }
            i++;
        }
        throw new common_1.NotFoundException(errors);
    }
};
exports.TypeormUtilsService = TypeormUtilsService;
exports.TypeormUtilsService = TypeormUtilsService = tslib_1.__decorate([
    tslib_1.__param(0, (0, common_1.Inject)(constants_1.CURRENT_ENTITY_REPOSITORY)),
    tslib_1.__metadata("design:paramtypes", [typeorm_1.Repository])
], TypeormUtilsService);
//# sourceMappingURL=typeorm-utils.service.js.map